package sun.management.snmp.jvminstr;

import com.sun.jmx.snmp.SnmpStatusException;
import java.util.Map;
import sun.management.snmp.util.JvmContextFactory;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryUsage;
import java.lang.management.MemoryType;
import javax.management.MBeanServer;
import com.sun.jmx.snmp.agent.SnmpMib;
import sun.management.snmp.util.MibLogger;
import sun.management.snmp.jvmmib.EnumJvmMemoryGCVerboseLevel;
import sun.management.snmp.jvmmib.EnumJvmMemoryGCCall;
import sun.management.snmp.jvmmib.JvmMemoryMBean;

public class JvmMemoryImpl implements JvmMemoryMBean
{
    static final EnumJvmMemoryGCCall JvmMemoryGCCallSupported;
    static final EnumJvmMemoryGCCall JvmMemoryGCCallStart;
    static final EnumJvmMemoryGCCall JvmMemoryGCCallFailed;
    static final EnumJvmMemoryGCCall JvmMemoryGCCallStarted;
    static final EnumJvmMemoryGCVerboseLevel JvmMemoryGCVerboseLevelVerbose;
    static final EnumJvmMemoryGCVerboseLevel JvmMemoryGCVerboseLevelSilent;
    static final String heapMemoryTag = "jvmMemory.getHeapMemoryUsage";
    static final String nonHeapMemoryTag = "jvmMemory.getNonHeapMemoryUsage";
    static final Long Long0;
    static final MibLogger log;
    
    public JvmMemoryImpl(SnmpMib snmpMib) {
    }
    
    public JvmMemoryImpl(SnmpMib snmpMib, MBeanServer mBeanServer) {
    }
    
    private MemoryUsage getMemoryUsage(MemoryType memoryType) {
        if (memoryType == MemoryType.HEAP) {
            return ManagementFactory.getMemoryMXBean().getHeapMemoryUsage();
        }
        return ManagementFactory.getMemoryMXBean().getNonHeapMemoryUsage();
    }
    
    MemoryUsage getNonHeapMemoryUsage() {
        try {
            Map<String, MemoryUsage> userData = JvmContextFactory.getUserData();
            if (userData != null) {
                MemoryUsage memoryUsage = (MemoryUsage)userData.get("jvmMemory.getNonHeapMemoryUsage");
                if (memoryUsage != null) {
                    JvmMemoryImpl.log.debug("getNonHeapMemoryUsage", "jvmMemory.getNonHeapMemoryUsage found in cache.");
                    return memoryUsage;
                }
                MemoryUsage memoryUsage2 = this.getMemoryUsage(MemoryType.NON_HEAP);
                userData.put("jvmMemory.getNonHeapMemoryUsage", memoryUsage2);
                return memoryUsage2;
            }
            else {
                JvmMemoryImpl.log.trace("getNonHeapMemoryUsage", "ERROR: should never come here!");
                return this.getMemoryUsage(MemoryType.NON_HEAP);
            }
        }
        catch (RuntimeException obj) {
            JvmMemoryImpl.log.trace("getNonHeapMemoryUsage", "Failed to get NonHeapMemoryUsage: " + obj);
            JvmMemoryImpl.log.debug("getNonHeapMemoryUsage", obj);
            throw obj;
        }
    }
    
    MemoryUsage getHeapMemoryUsage() {
        try {
            Map<String, MemoryUsage> userData = JvmContextFactory.getUserData();
            if (userData != null) {
                MemoryUsage memoryUsage = (MemoryUsage)userData.get("jvmMemory.getHeapMemoryUsage");
                if (memoryUsage != null) {
                    JvmMemoryImpl.log.debug("getHeapMemoryUsage", "jvmMemory.getHeapMemoryUsage found in cache.");
                    return memoryUsage;
                }
                MemoryUsage memoryUsage2 = this.getMemoryUsage(MemoryType.HEAP);
                userData.put("jvmMemory.getHeapMemoryUsage", memoryUsage2);
                return memoryUsage2;
            }
            else {
                JvmMemoryImpl.log.trace("getHeapMemoryUsage", "ERROR: should never come here!");
                return this.getMemoryUsage(MemoryType.HEAP);
            }
        }
        catch (RuntimeException obj) {
            JvmMemoryImpl.log.trace("getHeapMemoryUsage", "Failed to get HeapMemoryUsage: " + obj);
            JvmMemoryImpl.log.debug("getHeapMemoryUsage", obj);
            throw obj;
        }
    }
    
    public Long getJvmMemoryNonHeapMaxSize() throws SnmpStatusException {
        long max = this.getNonHeapMemoryUsage().getMax();
        if (max > -1L) {
            return new Long(max);
        }
        return JvmMemoryImpl.Long0;
    }
    
    public Long getJvmMemoryNonHeapCommitted() throws SnmpStatusException {
        long committed = this.getNonHeapMemoryUsage().getCommitted();
        if (committed > -1L) {
            return new Long(committed);
        }
        return JvmMemoryImpl.Long0;
    }
    
    public Long getJvmMemoryNonHeapUsed() throws SnmpStatusException {
        long used = this.getNonHeapMemoryUsage().getUsed();
        if (used > -1L) {
            return new Long(used);
        }
        return JvmMemoryImpl.Long0;
    }
    
    public Long getJvmMemoryNonHeapInitSize() throws SnmpStatusException {
        long init = this.getNonHeapMemoryUsage().getInit();
        if (init > -1L) {
            return new Long(init);
        }
        return JvmMemoryImpl.Long0;
    }
    
    public Long getJvmMemoryHeapMaxSize() throws SnmpStatusException {
        long max = this.getHeapMemoryUsage().getMax();
        if (max > -1L) {
            return new Long(max);
        }
        return JvmMemoryImpl.Long0;
    }
    
    public EnumJvmMemoryGCCall getJvmMemoryGCCall() throws SnmpStatusException {
        Map<Object, Object> userData = JvmContextFactory.getUserData();
        if (userData != null) {
            EnumJvmMemoryGCCall enumJvmMemoryGCCall = (EnumJvmMemoryGCCall)userData.get("jvmMemory.getJvmMemoryGCCall");
            if (enumJvmMemoryGCCall != null) {
                return enumJvmMemoryGCCall;
            }
        }
        return JvmMemoryImpl.JvmMemoryGCCallSupported;
    }
    
    public void setJvmMemoryGCCall(EnumJvmMemoryGCCall enumJvmMemoryGCCall) throws SnmpStatusException {
        if (enumJvmMemoryGCCall.intValue() == JvmMemoryImpl.JvmMemoryGCCallStart.intValue()) {
            Map<String, JvmMemoryImpl> userData = JvmContextFactory.getUserData();
            try {
                ManagementFactory.getMemoryMXBean().gc();
                if (userData != null) {
                    userData.put("jvmMemory.getJvmMemoryGCCall", JvmMemoryImpl.JvmMemoryGCCallStarted);
                }
            }
            catch (Exception ex) {
                if (userData != null) {
                    userData.put("jvmMemory.getJvmMemoryGCCall", JvmMemoryImpl.JvmMemoryGCCallFailed);
                }
            }
            return;
        }
        throw new SnmpStatusException(10);
    }
    
    public void checkJvmMemoryGCCall(EnumJvmMemoryGCCall enumJvmMemoryGCCall) throws SnmpStatusException {
        if (enumJvmMemoryGCCall.intValue() != JvmMemoryImpl.JvmMemoryGCCallStart.intValue()) {
            throw new SnmpStatusException(10);
        }
        return;
    }
    
    public Long getJvmMemoryHeapCommitted() throws SnmpStatusException {
        long committed = this.getHeapMemoryUsage().getCommitted();
        if (committed > -1L) {
            return new Long(committed);
        }
        return JvmMemoryImpl.Long0;
    }
    
    public EnumJvmMemoryGCVerboseLevel getJvmMemoryGCVerboseLevel() throws SnmpStatusException {
        if (ManagementFactory.getMemoryMXBean().isVerbose()) {
            return JvmMemoryImpl.JvmMemoryGCVerboseLevelVerbose;
        }
        return JvmMemoryImpl.JvmMemoryGCVerboseLevelSilent;
    }
    
    public void setJvmMemoryGCVerboseLevel(EnumJvmMemoryGCVerboseLevel enumJvmMemoryGCVerboseLevel) throws SnmpStatusException {
        if (JvmMemoryImpl.JvmMemoryGCVerboseLevelVerbose.intValue() == enumJvmMemoryGCVerboseLevel.intValue()) {
            ManagementFactory.getMemoryMXBean().setVerbose(true);
        }
        else {
            ManagementFactory.getMemoryMXBean().setVerbose(false);
        }
    }
    
    public void checkJvmMemoryGCVerboseLevel(EnumJvmMemoryGCVerboseLevel enumJvmMemoryGCVerboseLevel) throws SnmpStatusException {
    }
    
    public Long getJvmMemoryHeapUsed() throws SnmpStatusException {
        long used = this.getHeapMemoryUsage().getUsed();
        if (used > -1L) {
            return new Long(used);
        }
        return JvmMemoryImpl.Long0;
    }
    
    public Long getJvmMemoryHeapInitSize() throws SnmpStatusException {
        long init = this.getHeapMemoryUsage().getInit();
        if (init > -1L) {
            return new Long(init);
        }
        return JvmMemoryImpl.Long0;
    }
    
    public Long getJvmMemoryPendingFinalCount() throws SnmpStatusException {
        long n = (long)ManagementFactory.getMemoryMXBean().getObjectPendingFinalizationCount();
        if (n > -1L) {
            return new Long((long)(int)n);
        }
        return new Long(0L);
    }
    
    static {
        JvmMemoryGCCallSupported = new EnumJvmMemoryGCCall("supported");
        JvmMemoryGCCallStart = new EnumJvmMemoryGCCall("start");
        JvmMemoryGCCallFailed = new EnumJvmMemoryGCCall("failed");
        JvmMemoryGCCallStarted = new EnumJvmMemoryGCCall("started");
        JvmMemoryGCVerboseLevelVerbose = new EnumJvmMemoryGCVerboseLevel("verbose");
        JvmMemoryGCVerboseLevelSilent = new EnumJvmMemoryGCVerboseLevel("silent");
        Long0 = new Long(0L);
        log = new MibLogger(JvmMemoryImpl.class);
    }
}
